ArenaHistory.GameDialog = {}
local GameDialog = ArenaHistory.GameDialog
local TeamSort = ArenaHistory.TeamSort

GameDialog.frame = ArenaHistory_Game_Dialog
GameDialog.container = nil

function GameDialog:show(game)
  self:displayGame(game)
  self.frame:Show()
end

function GameDialog:hide()
  self.frame:Hide()
end

function GameDialog:createTeamHeader(score, mmr, label, container)
  local texture = container:CreateTexture()
  local header = container:CreateFontString(nil, nil, "GameFontNormal")
  header:SetJustifyH("LEFT")
  header:SetJustifyV("CENTER")
  header:SetHeight(30)
  header:SetTextColor(1, 1, 1, 1)
  if mmr ~= nil then
    header:SetText(string.format("  %s (%d MMR)", label, mmr))
  else
    header:SetText("  " .. label)
  end
  texture:SetAllPoints(header)
  if score == 1 then
    texture:SetTexture(0, 1, 0, 0.5)
  elseif score == -1 then
    texture:SetTexture(1, 0, 0, 0.5)
  else
    texture:SetTexture(0.5, 0.5, 0.5, 0.5)
  end
  texture:Show()
  header:Show()
  return header
end

GameDialog.PLAYER_WIDTH = 350
GameDialog.STAT_WIDTH = 100
GameDialog.STAT_HEIGHT = 40
GameDialog.ROW_WIDTH = GameDialog.PLAYER_WIDTH + 4 * GameDialog.STAT_WIDTH

function GameDialog:createStatLabel(text, container)
  local label = container:CreateFontString(nil, nil, "GameFontNormal")
  label:SetText(text)
  label:SetWidth(self.STAT_WIDTH)
  label:SetHeight(self.STAT_HEIGHT)
  label:SetJustifyV("CENTER")
  label:SetJustifyH("LEFT")
  label:Show()
  return label
end

function GameDialog:createSpec(player, size, container)
  local icon = container:CreateTexture()
  icon:SetWidth(size)
  icon:SetHeight(size)
  icon:SetNonBlocking(true)
  local cut = 0.08
  icon:SetTexCoord(cut, 1 - cut, cut, 1 - cut)
  if player.spec_id ~= nil then
    local iconPath = select(4,
        GetSpecializationInfoByID(player.spec_id))
    if iconPath ~= nil then
      icon:SetTexture(iconPath)
    end
  end
  icon:Show()
  return icon
end


function GameDialog:createPlayerStats(player, has_full_data, container)
  local stats = CreateFrame("Frame", nil, container)
  stats:SetWidth(self.ROW_WIDTH)
  stats:SetHeight(self.STAT_HEIGHT)

  local playerData = CreateFrame("Frame", nil, stats)
  playerData:SetWidth(self.PLAYER_WIDTH)
  playerData:SetHeight(self.STAT_HEIGHT)
  playerData:SetPoint("TOPLEFT", stats, "TOPLEFT")
  playerData:Show()

  local specSize = 32
  local spec = self:createSpec(player, specSize, playerData)
  spec:SetPoint("TOPLEFT", playerData, "TOPLEFT", 0,
      (specSize - self.STAT_HEIGHT) / 2)

  local name = playerData:CreateFontString(nil, nil, "GameFontNormal")
  name:SetText((player.name or "Unknown name") .. " - " ..
      (player.server or "Unknown server"))
  name:SetHeight(self.STAT_HEIGHT)
  name:SetJustifyV("CENTER")
  name:SetPoint("TOP", playerData, "TOP")
  name:SetPoint("LEFT", spec, "RIGHT", 5, 0)
  name:SetTextColor(1, 1, 1)
  name:Show()

  local align = playerData
  if has_full_data then
    local damage = self:createStatLabel(player.damage_done ~= nil
        and string.format("%d", player.damage_done) or "", stats)
    damage:SetPoint("TOP", playerData, "TOP")
    damage:SetPoint("LEFT", playerData, "RIGHT")
    damage:SetTextColor(1, 1, 1)

    local healing = self:createStatLabel(player.healing_done ~= nil
      and string.format("%d", player.healing_done) or "", stats)
    healing:SetPoint("TOP", damage, "TOP")
    healing:SetPoint("LEFT", damage, "RIGHT")
    healing:SetTextColor(1, 1, 1)

    local ratingString = ""
    if player.rating_change ~= nil and player.rating_change > 0 then
      ratingString = string.format("+%d", player.rating_change)
    elseif player.rating_change ~= nil then
      ratingString = string.format("%d", player.rating_change)
    end
    local ratingChange = self:createStatLabel(ratingString, stats)
    ratingChange:SetPoint("TOP", healing, "TOP")
    ratingChange:SetPoint("LEFT", healing, "RIGHT")
    ratingChange:SetTextColor(1, 1, 1)

    align = ratingChange
  end

  local timeInCCString = ""
  if player.time_in_cc ~= nil then
    timeInCCString = string.format("%02d:%02d",
        player.time_in_cc / 60, player.time_in_cc % 60) 
  end
  local timeInCC = self:createStatLabel(timeInCCString, stats)
  timeInCC:SetPoint("TOP", align, "TOP")
  timeInCC:SetPoint("LEFT", align, "RIGHT")
  timeInCC:SetTextColor(1, 1, 1)

  stats:Show()
  return stats
end

function GameDialog:createTeamStats(team, has_full_data, container)
  local stats = CreateFrame("Frame", nil, container)
  stats:SetWidth(self.ROW_WIDTH)
  stats:SetHeight((table.getn(team) + 1) * self.STAT_HEIGHT)
  
  local player = self:createStatLabel("Player", stats)
  player:SetWidth(self.PLAYER_WIDTH)
  player:SetPoint("TOPLEFT", stats, "TOPLEFT")

  local align = player
  if has_full_data then
    local damage = self:createStatLabel("Damage", stats)
    damage:SetPoint("TOP", player, "TOP")
    damage:SetPoint("LEFT", player, "RIGHT")

    local healing = self:createStatLabel("Healing", stats)
    healing:SetPoint("TOP", damage, "TOP")
    healing:SetPoint("LEFT", damage, "RIGHT")

    local ratingChange = self:createStatLabel("Rating", stats)
    ratingChange:SetPoint("TOP", healing, "TOP")
    ratingChange:SetPoint("LEFT", healing, "RIGHT")

    align = ratingChange
  end

  local timeInCC = self:createStatLabel("CC'ed", stats)
  timeInCC:SetPoint("TOP", align, "TOP")
  timeInCC:SetPoint("LEFT", align, "RIGHT")

  TeamSort.sort(team)
  for _, player in pairs(team) do
    local playerStats = self:createPlayerStats(player, has_full_data, stats)
    playerStats:SetPoint("TOP", align, "BOTTOM")
    playerStats:SetPoint("LEFT", stats, "LEFT")
    align = playerStats
  end

  stats:Show()
  return stats
end

function GameDialog:displayGame(game)
  if self.container ~= nil then
    self.container:SetParent(nil)
    self.container:Hide()
    self.container = nil
  end

  self.container = CreateFrame("Frame", nil, self.frame)
  self.container:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 30, -60)
  self.container:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", -30, 30)
  self.container:Show()

  local title = ArenaHistory.UI:createGame(game, self.container, false)
  title:SetPoint("TOPLEFT", self.container, "TOPLEFT")

  local score = game.score or nil
  local myTeamHeader = self:createTeamHeader(score, game.my_team_mmr,
      "Your team", self.container)
  myTeamHeader:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -20)
  myTeamHeader:SetPoint("RIGHT", self.container, "RIGHT")

  local myTeamStats = self:createTeamStats(game.my_team, game.has_seen_results,
      self.container)
  myTeamStats:SetPoint("TOPLEFT", myTeamHeader, "BOTTOMLEFT", 0, -10)

  local oppositeTeamHeader = self:createTeamHeader(-score,
      game.opposite_team_mmr, "Opposite team", self.container)
  oppositeTeamHeader:SetPoint("TOP", myTeamStats, "BOTTOM", 0, -20)
  oppositeTeamHeader:SetPoint("LEFT", self.container, "LEFT")
  oppositeTeamHeader:SetPoint("RIGHT", self.container, "RIGHT")

  local oppositeTeamStats = self:createTeamStats(game.opposite_team,
      game.has_seen_results, self.container)
  oppositeTeamStats:SetPoint("TOPLEFT", oppositeTeamHeader, "BOTTOMLEFT",
      0, -10)

  self.frame:SetWidth(100 + GameDialog.ROW_WIDTH)
  self.frame:SetHeight(242 + GameDialog.STAT_HEIGHT *
      (table.getn(game.my_team) + table.getn(game.opposite_team) + 2))
end
