local GameProto = ArenaHistory.GameProto
local PB = LibStub("pblua").compile(GameProto.ast)
local BASE64 = LibStub:GetLibrary("base64")

local function playerProtoToRaw(proto)
  local dump = {}
  local self = proto
  dump.id = self.id
  dump.name = self.name
  dump.server = self.server
  dump.race = self.race
  dump.faction = self.faction
  dump.spec_id = self.spec_id
  dump.damage_done = self.damage_done
  dump.healing_done = self.healing_done
  dump.rating_change = self.rating_change
  dump.time_in_cc = self.time_in_cc
  return dump
end

local function gameProtoToRaw(proto)
  local dump = {}
  local self = proto
  dump.season = self.season
  dump.score = self.score
  dump.has_seen_results = self.has_seen_results
  dump.run_time_s = self.run_time_s
  dump.is_rated = self.is_rated
  dump.team_size = self.team_size
  dump.localized_map_name = self.localized_map_name
  dump.start_timestamp = self.start_timestamp
  dump.my_team_mmr = self.my_team_mmr
  dump.opposite_team_mmr = self.opposite_team_mmr
  dump.region = self.region
  dump.addon_version = self.addon_version
  dump.first_died_id = self.first_died_id
  dump.my_team = {}
  for _, player in pairs(self.my_team) do
    table.insert(dump.my_team, playerProtoToRaw(player))
  end
  dump.opposite_team = {}
  for _, player in pairs(self.opposite_team) do
    table.insert(dump.opposite_team, playerProtoToRaw(player))
  end
  return dump
end

function ArenaHistory:handleMigration()
  if self.db.global.games_base64 ~= nil then
    ArenaHistory.logger:info("Migrating base64 to raw")
    for _, base64 in pairs(self.db.global.games_base64) do
      local game = PB.Game()
      game:ParsePartial(BASE64.dec(base64))
      table.insert(self.db.global.games_raw, gameProtoToRaw(game))
    end
    self.db.global.games_base64 = nil
    ArenaHistory.logger:info("Finished migration base64 to raw")
  end
end


